@@ -5,9 +5,10 @@ DECLARE_CALL(compute_eafdiff_polygon_C, SEXP DATA, SEXP CUMSIZES, SEXP INTERVALS
 DECLARE_CALL(compute_eafdiff_rectangles_C, SEXP DATA, SEXP CUMSIZES, SEXP INTERVALS)
 DECLARE_CALL(R_read_datasets, SEXP FILENAME)
 DECLARE_CALL(hypervolume_C, SEXP DATA, SEXP REFERENCE)
-DECLARE_CALL(hv_contributions_C, SEXP DATA, SEXP REFERENCE)
+DECLARE_CALL(hv_contributions_C, SEXP DATA, SEXP REFERENCE, SEXP IGNORE_DOMINATED)
 DECLARE_CALL(normalise_C, SEXP DATA, SEXP RANGE, SEXP LBOUND, SEXP UBOUND, SEXP MAXIMISE)
 DECLARE_CALL(is_nondominated_C, SEXP DATA, SEXP MAXIMISE, SEXP KEEP_WEAKLY)
+DECLARE_CALL(any_dominated_C, SEXP DATA, SEXP MAXIMISE)
 DECLARE_CALL(pareto_ranking_C, SEXP DATA)
 DECLARE_CALL(epsilon_mul_C, SEXP DATA, SEXP REFERENCE, SEXP MAXIMISE)
 DECLARE_CALL(epsilon_add_C, SEXP DATA, SEXP REFERENCE, SEXP MAXIMISE)
