@@ -9,4 +9,9 @@
 #define __USE_MINGW_ANSI_STDIO 1
 #endif
 
+// Silence deprecation warnings from Clang and MSVC on Windows
+#if defined(_MSC_VER) || (defined(__clang__) && defined(_WIN32))
+# define _CRT_SECURE_NO_WARNINGS 1
+#endif
+
 #endif // _MOOCORE_CONFIG_H_
