@@ -55,6 +55,7 @@
 #include <stdint.h>
 #include "common.h"
 #include "pow_int.h"
+#include "nondominated.h" // minmax_from_bool
 
 static inline double
 gd_common (int dim, const signed char * restrict minmax,
@@ -135,7 +136,8 @@ IGD (const double * restrict data, int nobj, int npoints,
      const double * restrict ref, int ref_size,
      const bool *  restrict maximise)
 {
-    const signed char *minmax = minmax_from_bool(nobj, maximise);
+    ASSUME(nobj > 0 && nobj <= 128);
+    const signed char *minmax = minmax_from_bool(maximise, (dimension_t) nobj);
     double value = IGD_minmax (nobj, minmax, data, npoints, ref, ref_size);
     free ((void *)minmax);
     return value;
@@ -178,7 +180,8 @@ _attr_maybe_unused static double
 IGD_plus (const double * restrict data, int nobj, int npoints, const double * restrict ref, int ref_size,
           const bool * restrict  maximise)
 {
-    const signed char *minmax = minmax_from_bool(nobj, maximise);
+    ASSUME(nobj > 0 && nobj <= 128);
+    const signed char *minmax = minmax_from_bool(maximise, (dimension_t) nobj);
     double value = IGD_plus_minmax (nobj, minmax, data, npoints, ref, ref_size);
     free ((void *)minmax);
     return(value);
@@ -208,7 +211,8 @@ avg_Hausdorff_dist (const double * restrict data, int nobj, int npoints,
                     const double * restrict ref, int ref_size,
                     const bool * restrict maximise, unsigned int p)
 {
-    const signed char * minmax = minmax_from_bool(nobj, maximise);
+    ASSUME(nobj > 0 && nobj <= 128);
+    const signed char * minmax = minmax_from_bool(maximise, (dimension_t) nobj);
     double value = avg_Hausdorff_dist_minmax (nobj, minmax, data, npoints, ref, ref_size, p);
     free ((void *)minmax);
     return(value);
